import java.io.*;
import java.security.*;
import java.security.spec.*;
import java.security.interfaces.*;
import javax.crypto.*;

public class UPServerHandShaking {

    private ObjectInputStream   ois;
    private ObjectOutputStream  oos;
    private PublicKey           upserverPublicKey;
    private PublicKey           clientPublicKey;
    private PrivateKey          upserverPrivateKey;
    private SecretKey           secretKey;
    private UPServerKeyPair  ckp;

    public UPServerHandShaking(ObjectInputStream in, ObjectOutputStream out,
                                  UPServerKeyPair ck) {
        ois = in;
        oos = out;
        ckp = ck;
    }

    public boolean init() {
        try {
            // Initialize the DH protocol and exchange the secret key for cipher
            UPServerDHKeyAgreement dhKey = new UPServerDHKeyAgreement(ois, oos);
            secretKey = dhKey.agree();

            upserverPublicKey = ckp.getPublicKey();
            upserverPrivateKey = ckp.getPrivateKey();
            byte[] upserverEncodedPublicKey = ckp.getEncodedPublicKey();

            // System.err.println("UPServer: " + toHexString(upserverEncodedPublicKey));

            // send UPServer's public key to Client
            oos.writeObject(upserverEncodedPublicKey);
            oos.flush();
        
            // obtain Client's public key
            // This will be replaced by reading from client's certificate

            byte[] clientEncodedPublicKey = (byte[])ois.readObject();

            // System.err.println("Client's Encoded PublicKey: " + toHexString(clientEncodedPublicKey));

            KeyFactory upserverKeyFac = KeyFactory.getInstance("DSA");
            X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(clientEncodedPublicKey);
            clientPublicKey = upserverKeyFac.generatePublic(x509KeySpec);

            return true;
        } catch (Exception ex) {
            System.err.println("UPServerHandShaking.init() Exception-> " + ex);
            return false;
        }
    }

    public PublicKey getUPServerPublicKey() {
        return this.upserverPublicKey;
    }

    public PublicKey getClientPublicKey() {
        return this.clientPublicKey;
    }

    public PrivateKey getUPServerPrivateKey() {
        return this.upserverPrivateKey;
    }

    public SecretKey getSecretKey() {
        return this.secretKey;
    }

    public void byte2hex(byte b, StringBuffer buf) {
        char hexChars[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};

        int low = (b & 0x0f);
        int high = (b & 0xf0) >> 4;

        buf.append(hexChars[high]);
        buf.append(hexChars[low]);
    }

    /*
     * Converts a byte array to hex string
     */
    public String toHexString(byte[] block) {
        StringBuffer buf = new StringBuffer();

        int len = block.length;

        for (int i = 0; i < len; i++) {
             byte2hex(block[i], buf);
             if (i < len-1) {
                 buf.append(":");
             }
        }
        return buf.toString();
    }
}
